package de.bsvrz.vew.ereigniskal.ereigniskal.ereignis;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.List;
import java.util.Vector;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ClientApplication;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.dambach.vewdynobj.VerwaltungDynObj;
import de.bsvrz.vew.ereigniskal.ereigniskal.erinnerungsfunktion.Sender;
import de.bsvrz.vew.syskal.syskal.systemkalendereintrag.SystemkalenderArbeiter;

public class FeiertagTest implements ClientReceiverInterface
{

  /**
   * Datenverteilerverbindung.
   */
  private static ClientDavInterface _connection = null;

  /**
   * Pid Kalender
   */
  private static String _strKalender = "kv.aoe.nw.nba.koeln.koblenz";

  /**
   * Pid Konfigurationsbereich
   */
  private static String _strKonfigBereich = "kb.default.aoe.nw.nba.koeln.koblenz";

  private static Process _prozess = null;

  private static Vector<ResultData> _resultList = null;

  @BeforeClass
  public static void setUpBeforeClass() throws Exception
  {
    ClientDavParameters parameters = new ClientDavParameters();
    parameters.setDavCommunicationAddress("localhost");
    parameters.setDavCommunicationSubAddress(8083);
    parameters.setUserName("Tester");
    parameters.setUserPassword("geheim");

    _connection = new ClientDavConnection(parameters);
    _connection.connect();
    _connection.login();

    // System.out.println("Erzeuge SystemKalenderEintrge");
    
    erzeugeSystemKalenderEintrag("ske.montag", "Montag", "Montag");
    erzeugeSystemKalenderEintrag("ske.dienstag", "Dienstag", "Dienstag");
    erzeugeSystemKalenderEintrag("ske.mittwoch", "Mittwoch", "Mittwoch");
    erzeugeSystemKalenderEintrag("ske.donnerstag", "Donnerstag", "Donnerstag");
    erzeugeSystemKalenderEintrag("ske.freitag", "Freitag", "Freitag");
    erzeugeSystemKalenderEintrag("ske.samstag", "Samstag", "Samstag");
    erzeugeSystemKalenderEintrag("ske.sonntag", "Sonntag", "Sonntag");
    erzeugeSystemKalenderEintrag("ske.ostersonntag", "Ostersonntag", "Ostersonntag");
    erzeugeSystemKalenderEintrag("ske.tag", "Tag", "Tag");
    erzeugeSystemKalenderEintrag("ske.neujahr", "Neujahr", "Neujahr:=01.01.*,*");
    erzeugeSystemKalenderEintrag("ske.karfreitag", "Karfreitag", "Karfreitag:=Ostersonntag-2Tage");
    erzeugeSystemKalenderEintrag("ske.tdde", "Einheit", "Einheit:=03.10.*,*");
    erzeugeSystemKalenderEintrag("ske.tda", "Arbeit", "Arbeit:=01.05.*,*");
    erzeugeSystemKalenderEintrag("ske.allerheil", "Allerheil", "Allerheil:=01.11.*,*");
    erzeugeSystemKalenderEintrag("ske.weihnacht1", "Weihnacht1", "Weihnacht1:=25.12.*,*");
    erzeugeSystemKalenderEintrag("ske.weihnacht2", "Weihnacht2", "Weihnacht2:=26.12.*,*");
    erzeugeSystemKalenderEintrag("ske.ostermontag", "Ostermontag", "Ostermontag:=Ostersonntag+1Tag");
    erzeugeSystemKalenderEintrag("ske.himmel", "Himmel", "Himmel:=Ostersonntag+39Tage");
    erzeugeSystemKalenderEintrag("ske.pfingstmo", "PfingstMo", "PfingstMo:=Ostersonntag+50Tage");
    erzeugeSystemKalenderEintrag("ske.fronleich", "Fronleich", "Fronleich:=Ostersonntag+60Tage");
    
    
    erzeugeSystemKalenderEintrag("ske.feiertag", "Feiertag", "Feiertag:=ODER{Neujahr,Karfreitag,Ostersonntag,Ostermontag,Arbeit,Himmel,PfingstMo,Fronleich,Einheit,Allerheil,Weihnacht1,Weihnacht2}*,*");
    erzeugeSystemKalenderEintrag("ske.mobissa", "MoBisSa", "MoBisSa:=ODER{Montag,Dienstag,Mittwoch,Donnerstag,Freitag,Samstag}*,*");
    erzeugeSystemKalenderEintrag("ske.mobisfr", "MoBisFr", "MoBisFr:=ODER{Montag,Dienstag,Mittwoch,Donnerstag,Freitag}*,*");
    
    erzeugeSystemKalenderEintrag("ske.werktag1", "Werktag1", "Werktag:=UND{MoBisSa,NICHT Feiertag}*,*");
    erzeugeSystemKalenderEintrag("ske.werktag2", "Werktag2", "Werktag:=UND{MoBisFr,NICHT Feiertag}*,*");
    
    erzeugeEreignisTyp("ereignisTyp.tag", "EreignisTypTag", 10);
    erzeugeEreignisTyp("ereignisTyp.werktag", "EreignisTypWerktag", 10);
    

    // System.out.println("Erzeuge Ereignisse");

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");
    Data data = _connection.createData(atg);
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date date1 = null;
    Date date2 = null;
    try
    {
      date1 = sdf.parse("01.01.2004 00:00:00,000");
      date2 = sdf.parse("01.01.2004 00:00:00,000");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
   
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    SystemObject so = _connection.getDataModel().getObject("ske.montag");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.montag", "Montag", "ereignisTyp.tag", data);

    data = _connection.createData(atg);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.dienstag");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.dienstag", "Dienstag", "ereignisTyp.tag", data);
    
    data = _connection.createData(atg);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.mittwoch");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.mittwoch", "Mittwoch", "ereignisTyp.tag", data);
    
    data = _connection.createData(atg);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.donnerstag");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.donnerstag", "Donnerstag", "ereignisTyp.tag", data);
    
    data = _connection.createData(atg);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.freitag");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.freitag", "Freitag", "ereignisTyp.tag", data);
    
    data = _connection.createData(atg);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.samstag");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.samstag", "Samstag", "ereignisTyp.tag", data);
    
    data = _connection.createData(atg);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.sonntag");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.sonntag", "Sonntag", "ereignisTyp.tag", data);
    
    data = _connection.createData(atg);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.werktag1");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.werktag1", "Werktag1", "ereignisTyp.tag", data);
    
    data = _connection.createData(atg);
    data.getTimeValue("BeginnZeitlicheGltigkeit").setMillis(date1.getTime());
    data.getTimeValue("EndeZeitlicheGltigkeit").setMillis(date2.getTime());
    so = _connection.getDataModel().getObject("ske.werktag2");
    data.getItem("SystemKalenderEintragReferenz").asReferenceValue().setSystemObject(so);
    data.getTextValue("Quelle").setText("");
    erzeugeEreignis("ereignis.werktag2", "Werktag2", "ereignisTyp.tag", data);
    
    new FeiertagTest().anmeldenEreignisAntwort();

//    starteAppliaktionsProzess();

    System.out.println("Warten bis Applikation gestartet...");

    Thread.sleep(20000);

  }

  private static void starteAppliaktionsProzess() throws IOException
  {
    // String separator = System.getProperty("file.separator");
    String separator = "/";
    String javaHome = System.getProperty("java.home");
    String classPath = System.getProperty("java.class.path");

    ClientDavParameters params = _connection.getClientDavParameters();
    String dav = params.getDavCommunicationAddress() + ":" + params.getDavCommunicationSubAddress();
    String user = params.getUserName();

    String cmdLine = javaHome + separator + "bin" + separator + "java";
    cmdLine += " -cp " + "\"" + classPath + "\"" + " -Xmx128m ";
    cmdLine += "de.bsvrz.vew.ereigniskal.ereigniskal.EreignisKalender";
    cmdLine += " -datenverteiler=" + dav;
    cmdLine += " -benutzer=" + user;
    cmdLine += " -authentifizierung=passwd";
    // cmdLine += " -authentifizierung=C:/kernsoftware-3.4.6-test/skripte-dosshell/passwd";
    cmdLine += " -debugLevelStdErrText=CONFIG";
    cmdLine += " -kalender=" + _connection.getLocalConfigurationAuthority().getPid();
    cmdLine += " -neuberechnung=" + "\"" + "01.02.2008 01:00:00,000" + "\"";

    _prozess = Runtime.getRuntime().exec(cmdLine);

    new StdErrCatcher().start();

  }

  private static class StdErrCatcher extends Thread
  {

    byte[] buffer = new byte[1024];

    @Override
    public void run()
    {
      while (true)
      {

        try
        {
          if (_prozess.getErrorStream().available() != 0)
          {
            int anzahl = _prozess.getErrorStream().read(buffer);
            System.out.print("" + new String(buffer, 0, anzahl));

          }
          else
          {
            sleep(100);
          }
        }
        catch (IOException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace(System.out);
        }
        catch (InterruptedException e)
        {
          // TODO Auto-generated catch block
          e.printStackTrace(System.out);
        }

      }
    }
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception
  {
    if (_prozess != null)
      _prozess.destroy();
    
    loescheDynamischesObjekt("ereignis.montag", "Ereignisse");
    loescheDynamischesObjekt("ereignis.dienstag", "Ereignisse");
    loescheDynamischesObjekt("ereignis.mittwoch", "Ereignisse");
    loescheDynamischesObjekt("ereignis.donnerstag", "Ereignisse");
    loescheDynamischesObjekt("ereignis.freitag", "Ereignisse");
    loescheDynamischesObjekt("ereignis.samstag", "Ereignisse");
    loescheDynamischesObjekt("ereignis.sonntag", "Ereignisse");
    loescheDynamischesObjekt("ereignis.werktag1", "Ereignisse");
    loescheDynamischesObjekt("ereignis.werktag2", "Ereignisse");
         
    loescheDynamischesObjekt("ereignisTyp.tag", "EreignisTypen");
    loescheDynamischesObjekt("ereignisTyp.werktag", "EreignisTypen");
      
    loescheDynamischesObjekt("ske.montag", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.dienstag", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.mittwoch",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.donnerstag",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.freitag",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.samstag",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.sonntag",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.ostersonntag",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.tag",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.neujahr", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.karfreitag",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.tdde", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.tda",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.allerheil",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.weihnacht1",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.weihnacht2",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.ostermontag",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.himmel",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.pfingstmo",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.fronleich",  "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.feiertag", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.mobissa", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.mobisfr", "SystemKalenderEintrge");    
    loescheDynamischesObjekt("ske.werktag1", "SystemKalenderEintrge");
    loescheDynamischesObjekt("ske.werktag2",  "SystemKalenderEintrge");
   }

  @Before
  public void setUp() throws Exception
  {
  }

  @After
  public void tearDown() throws Exception
  {
  }

  private void anmeldenEreignisAntwort()
  {

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisKalenderAntwort");
    Aspect asp = _connection.getDataModel().getAspect("asp.antwort");
    DataDescription dd = new DataDescription(atg, asp);

    ClientApplication lao = _connection.getLocalApplicationObject();
    _connection.subscribeReceiver(this, (SystemObject)lao, dd, ReceiveOptions.normal(), ReceiverRole.drain());
  }

  @Test(timeout = 5000)
  public void feiertagTest1() throws Exception
  {

    // anmeldenEreignisAntwort();

    AttributeGroup atg = _connection.getDataModel().getAttributeGroup("atg.ereignisKalenderAnfrage");
    Aspect asp = _connection.getDataModel().getAspect("asp.anfrage");
    DataDescription dd = new DataDescription(atg, asp);

    Data data = _connection.createData(atg);
    ClientApplication lao = _connection.getLocalApplicationObject();

    data.getItem("absenderId").asReferenceValue().setSystemObject((SystemObject)lao);
    data.getItem("absenderZeichen").asTextValue().setText("Anfrage 1");
    SimpleDateFormat sdf = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss,SSS");
    Date date1 = null;
    Date date2 = null;
    try
    {
      date1 = sdf.parse("03.08.2014 00:00:00,000");
      date2 = sdf.parse("04.08.2014 00:00:00,000");
    }
    catch (ParseException e)
    {
      // TODO Auto-generated catch block
      e.printStackTrace();
    }
    data.getItem("Anfangszeitpunkt").asTimeValue().setMillis(date1.getTime());
    data.getItem("Endzeitpunkt").asTimeValue().setMillis(date2.getTime());
    data.getItem("EreignisTypenOption").asTextValue().setText("Alle");

    ResultData rd = new ResultData(_connection.getLocalConfigurationAuthority(), dd, new Date().getTime(), data);
    Sender sender = new Sender(_connection, SenderRole.sender());

    _resultList = new Vector<ResultData>();

    sender.sende(rd, dd);

    // Warten bis EreignisKalender die Antwort publiziert hat
    synchronized (_resultList)
    {
      if (_resultList.size() < 1)
      {
        _resultList.wait();
      }
    }

    ResultData resultData = _resultList.get(0);

    data = resultData.getData();

    String s = data.getItem("absenderZeichen").asTextValue().getText();

    assertEquals("Test 1", "Anfrage 1", s);

    s = data.getItem("nderung").asTextValue().getText();

    assertEquals("Test 2", "Nein", s);

    Array array = data.getItem("Ereignis").asArray();

    String[] strings = new String[array.getLength()];

    for (int i = 0; i < array.getLength(); i++)
    {
      Data data2 = array.getItem(i);

      long l = data2.getItem("Zeitpunkt").asTimeValue().getMillis();
      Date d = new Date();
      d.setTime(l);
      String s1 = sdf.format(d);

      String s2 = data2.getItem("EreignisReferenz").asReferenceValue().getSystemObject().getPid();
      String s3 = data2.getItem("zeitlichGltig").asTextValue().getText();
      String s4 = data2.getItem("verkehrlichGltig").asTextValue().getText();

      strings[i] = s1 + " " + s2 + " " + s3 + " " + s4;

      System.out.println(strings[i]);

    }

    System.out.println(array.getLength());

    assertTrue("Test 3", array.getLength() == 5);    
    assertEquals("Test 4", "03.08.2014 00:00:00,000 ereignis.sonntag Ja Nein", strings[0]);
    assertEquals("Test 5", "04.08.2014 00:00:00,000 ereignis.sonntag Nein Nein", strings[1]);
    assertEquals("Test 6", "04.08.2014 00:00:00,000 ereignis.werktag2 Ja Nein", strings[2]);
    assertEquals("Test 7", "04.08.2014 00:00:00,000 ereignis.werktag1 Ja Nein", strings[3]);
    assertEquals("Test 9", "04.08.2014 00:00:00,000 ereignis.montag Ja Nein", strings[4]);
  }

  private static void erzeugeSystemKalenderEintrag(String pid, String name, String definition)
  {

    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(_strKalender);
    DynamicObjectType dot = (DynamicObjectType)_connection.getDataModel().getType("typ.systemKalenderEintrag");

    // Erzeuge dynamisches Objekt
    AttributeGroup atgKonfig = _connection.getDataModel().getAttributeGroup("atg.systemKalenderEintrag");
    Aspect aspKonfig = _connection.getDataModel().getAspect("asp.eigenschaften");
    VerwaltungDynObj vewKonfig = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgKonfig,
        aspKonfig);

    Data data = _connection.createData(atgKonfig);
    Data[] datas = new Data[1];
    datas[0] = data;
    vewKonfig.erzeuge(pid, name, "SystemKalenderEintrge", null);

    // Erzeuge Parameter
    AttributeGroup atgParam = _connection.getDataModel().getAttributeGroup("atg.systemKalenderEintrag");
    Aspect aspParam = _connection.getDataModel().getAspect("asp.parameterVorgabe");
    VerwaltungDynObj vewParam = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgParam,
        aspParam);
    data = _connection.createData(atgParam);
    data.getTextValue("Definition").setText(definition);
    datas[0] = data;
    vewParam.setDynamicObject((DynamicObject)_connection.getDataModel().getObject(pid));
    vewParam.parametriere(datas);
  }

  private static void erzeugeEreignisTyp(String pid, String name, int prio)
  {

    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(_strKalender);
    DynamicObjectType dot = (DynamicObjectType)_connection.getDataModel().getType("typ.ereignisTyp");

    // Erzeuge dynamisches Objekt
    AttributeGroup atgKonfig = _connection.getDataModel().getAttributeGroup("atg.ereignisTypEigenschaften");
    Aspect aspKonfig = _connection.getDataModel().getAspect("asp.eigenschaften");
    VerwaltungDynObj vewKonfig = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgKonfig,
        aspKonfig);

    Data data = _connection.createData(atgKonfig);
    Array array = data.getItem("ZustzlicheAttribute").asArray();
    array.setLength(1);
    Data d = array.getItem(0);
    d.getTextValue("Attributname").setText("");
    d.getTextValue("Attributwert").setText("");
    Data[] datas = new Data[1];
    datas[0] = data;
    vewKonfig.erzeuge(pid, name, "EreignisTypen", datas);

    // Erzeuge Parameter
    AttributeGroup atgParam = _connection.getDataModel().getAttributeGroup("atg.ereignisTypParameter");
    Aspect aspParam = _connection.getDataModel().getAspect("asp.parameterVorgabe");
    VerwaltungDynObj vewParam = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgParam,
        aspParam);
    data = _connection.createData(atgParam);
    data.getScaledValue("EreignisTypPrioritt").set(prio);
    datas[0] = data;
    vewParam.setDynamicObject((DynamicObject)_connection.getDataModel().getObject(pid));
    vewParam.parametriere(datas);

  }

  private static void erzeugeEreignis(String pid, String name, String ereignisTyp, Data parameter)
  {

    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(_strKalender);
    DynamicObjectType dot = (DynamicObjectType)_connection.getDataModel().getType("typ.ereignis");

    // Erzeuge dynamisches Objekt
    AttributeGroup atgKonfig = _connection.getDataModel().getAttributeGroup("atg.ereignisEigenschaften");
    Aspect aspKonfig = _connection.getDataModel().getAspect("asp.eigenschaften");
    VerwaltungDynObj vewKonfig = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgKonfig,
        aspKonfig);

    Data data = _connection.createData(atgKonfig);
    data.getItem("Ereignisbeschreibung").asTextValue().setText("");
    SystemObject so = _connection.getDataModel().getObject(ereignisTyp);
    data.getItem("EreignisTypReferenz").asReferenceValue().setSystemObject(so);
    Data[] datas = new Data[1];
    datas[0] = data;
    vewKonfig.erzeuge(pid, name, "Ereignisse", datas);

    // Erzeuge Parameter
    AttributeGroup atgParam = _connection.getDataModel().getAttributeGroup("atg.ereignisParameter");
    Aspect aspParam = _connection.getDataModel().getAspect("asp.parameterVorgabe");
    VerwaltungDynObj vewParam = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgParam,
        aspParam);

    datas[0] = parameter;
    vewParam.setDynamicObject((DynamicObject)_connection.getDataModel().getObject(pid));
    vewParam.parametriere(datas);
  }

  private static void loescheDynamischesObjekt(String pid, String menge)
  {
    ConfigurationArea ca = _connection.getDataModel().getConfigurationArea(_strKonfigBereich);
    ConfigurationObject co = (ConfigurationObject)_connection.getDataModel().getObject(_strKalender);
    DynamicObjectType dot = null;
    AttributeGroup atgKonfig = null;
    if (menge.equals("SystemKalenderEintrge"))
    {
      dot = (DynamicObjectType)_connection.getDataModel().getType("typ.systemKalenderEintrag");
      atgKonfig = _connection.getDataModel().getAttributeGroup("atg.systemKalenderEintrag");
    }
    else if (menge.equals("EreignisTypen"))
    {
      dot = (DynamicObjectType)_connection.getDataModel().getType("typ.ereignisTyp");
      atgKonfig = _connection.getDataModel().getAttributeGroup("atg.ereignisTypEigenschaften");
    }
    else if (menge.equals("Ereignisse"))
    {
      dot = (DynamicObjectType)_connection.getDataModel().getType("typ.ereignis");
      atgKonfig = _connection.getDataModel().getAttributeGroup("atg.ereignisEigenschaften");
    }
    else
      fail("Falscher Typ");

    Aspect aspKonfig = _connection.getDataModel().getAspect("asp.eigenschaften");
    VerwaltungDynObj vewKonfig = new VerwaltungDynObj(_connection, _connection.getDataModel(), ca, dot, co, atgKonfig,
        aspKonfig);

    vewKonfig.setDynamicObject(_connection.getDataModel().getObject(pid));
    vewKonfig.loesche(pid, menge);
  }

  public void update(ResultData[] results)
  {
    // TODO Auto-generated method stub
    for (ResultData result : results)
    {
      if (result.getData() != null)
      {

        DataDescription dd = result.getDataDescription();
        AttributeGroup atg = dd.getAttributeGroup();

        if (atg.getPid().equals("atg.ereignis") || atg.getPid().equals("atg.ereignisKalenderAntwort"))
        {

//          System.out.println("OK: " + result.getObject() + " " + result.getDataDescription());

          synchronized (_resultList)
          {
            _resultList.add(result);
            _resultList.notify();
          }
        }
        else
        {
//          System.out.println("Atg: " + result.getObject() + " " + result.getDataDescription());
        }

      }
      else
      {
//        System.out.println("Null: " + result.getObject() + " " + result.getDataDescription());
      }
    }
  }

}
