/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.ereigniskal.ereigniskal.erinnerungsfunktion;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;
import java.util.Timer;

import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.ereigniskal.ereigniskal.NeuBerechnung;
import de.bsvrz.vew.ereigniskal.ereigniskal.ereignis.Ereignis;

/**
 * Klasse stellt einen Wecker dar
 * 
 * @version $Revision: 1.4 $ / $Date: 2015/06/08 15:13:13 $ / ($Author: Pittner $)
 * @author Dambach-Werke GmbH
 * @author Timo Pittner
 */
public class ErinnerungsFunktion
{

  /**
   * Das Timer Objekt
   */
  private Timer timer = new Timer();

  /**
   * Das Ereignis, wird dem Sender-Instanz bergeben
   */
  private Ereignis e;

  /**
   * Das Ereignis, wird dem Sender-Instanz bergeben
   */
  private NeuBerechnung n;

  /**
   * Der Debugger, stellt verschiedene Protokollierungsfunktionen zur Verfgung
   */
  private Debug _debug;

  /**
   * Konstruktor der Klasse
   * 
   * @param e
   *          das Ereignis, welches den Zustand meldet
   * @param von
   *          Anfangszeitpunkt
   * @param bis
   *          Endzeitpunkt
   */
  public ErinnerungsFunktion(Ereignis e, Long von, Long bis)
  {
    this.e = e;

    _debug = Debug.getLogger();

    for (Map.Entry<Long, Boolean> me : e.getListeZustandsWechselZeit().getListeZustandsWechsel().entrySet())
    {
      if (me.getKey() >= von && me.getKey() < bis)
      {
        Long l = me.getKey();
        load(l);
      }

    }

  }

  /**
   * Konstruktor der Klasse
   * 
   * @param e
   *          das Ereignis, welches den Zustand meldet
   * @param b
   *          Schalter, der den Wecker sofort(false) oder zum definerten Zeitpunkt(true) losgehen laesst
   */
  public ErinnerungsFunktion(Ereignis e, Boolean b)
  {
    this.e = e;

    _debug = Debug.getLogger();

    Long now = new Date().getTime();

    if (b)
    {

      for (Map.Entry<Long, Boolean> me : e.getListeZustandsWechselZeit().getListeZustandsWechsel().entrySet())
      {
        if (me.getKey() > now)
        {

          Long l = me.getKey();
          load(l);

        }
      }

      for (Map.Entry<Long, Boolean> me : e.getListeZustandsWechselVerkehr().getListeZustandsWechsel().entrySet())
      {
        if (me.getKey() > now)
        {

          Long l = me.getKey();
          load(l);

        }
      }
    }
    else
    {

      load(now);

    }

  }

  /**
   * Konstruktor der Klasse
   * 
   * @param e
   *          das Ereignis, welches den Zustand meldet
   * @param b
   *          Schalter, der den Wecker sofort(false) oder zum definerten Zeitpunkt(true) losgehen laesst
   * @param c
   *          Schalter, der den Zustand der vzeitl. und verkehrlichen Gltigkeit setzt
   */
  public ErinnerungsFunktion(Ereignis e, Boolean b,  Boolean c)
  {
    this.e = e;

    _debug = Debug.getLogger();

    Long now = new Date().getTime();

    if (b)
    {

      for (Map.Entry<Long, Boolean> me : e.getListeZustandsWechselZeit().getListeZustandsWechsel().entrySet())
      {
        if (me.getKey() > now)
        {

          Long l = me.getKey();
          load(l);

        }
      }

      for (Map.Entry<Long, Boolean> me : e.getListeZustandsWechselVerkehr().getListeZustandsWechsel().entrySet())
      {
        if (me.getKey() > now)
        {

          Long l = me.getKey();
          load(l);

        }
      }
    }
    else
    {

      load(now ,c);

    }

  }
  
  
  
  /**
   * Konstruktor der Klasse
   * 
   * @param e
   *          das Ereignis, welches den Zustand meldet
   * @param b
   *          Schalter, der den Wecker sofort(false) oder zum definerten Zeitpunkt(true) losgehen laesst
   */
  public ErinnerungsFunktion(Ereignis e)
  {
    this.e = e;

    _debug = Debug.getLogger();

    Long now = new Date().getTime();

    for (Map.Entry<Long, Boolean> me : e.getListeZustandsWechselZeit().getListeZustandsWechsel().entrySet())
    {
      if (me.getKey() > now)
      {

        Long l = me.getKey();
        load(l);

      }
    }

    for (Map.Entry<Long, Boolean> me : e.getListeZustandsWechselVerkehr().getListeZustandsWechsel().entrySet())
    {
      if (me.getKey() > now)
      {

        Long l = me.getKey();
        load(l);

      }
    }
   
  }
  
  
  /**
   * stellt den Wecker
   * 
   * @param l
   *          Weckzeit
   */
  private void load(Long l)
  {
    Date date = new Date();
    date.setTime(l);
    ResultData result = e.berechneDatenEreignisZustand(l);
    SenderZeitgesteuert s1 = new SenderZeitgesteuert(result);
    
    timer.schedule(s1, date);

  }
  /**
   * stellt den Wecker
   * 
   * @param l
   *          Weckzeit
   */
  private void load(Long l, Boolean b)
  {
    Date date = new Date();
    date.setTime(l);
    ResultData result = e.setzteDatenEreignisZustand(b);    
    SenderZeitgesteuert s1 = new SenderZeitgesteuert(result);
    
    timer.schedule(s1, date);
    
  }

  /**
   * loescht den Timer
   * 
   */
  public void unload()
  {

    timer.cancel();
    
  }
}